@@ -7,6 +7,13 @@ from json import load
 from transparentpath import TransparentPath
 
 
+def handle_path(path, fs):
+    if fs is not None:
+        return TransparentPath(path, fs=fs)
+    else:
+        return TransparentPath(path)
+
+
 class Config:
 
     # noinspection PyUnresolvedReferences
@@ -49,8 +56,13 @@ class Config:
         path: Optional[Union[str, Path, TransparentPath]]
             The path to the json file containing the configuration. Will be casted into a transparentpath if a str is
             given, so make sure that if you pass a str that should be local, you did not set a global file system.
+            If None, creates an empty configuration.
         """
 
+        if path is None:
+            self.configs = {}
+            return
+
         if type(path) == str:
             path = TransparentPath(path)
 
@@ -58,7 +70,7 @@ class Config:
             raise ValueError("Config path must be a json")
 
         if not path.is_file():
-            raise FileNotFoundError(f"No file found for node configuration {self.path}")
+            raise FileNotFoundError(f"No file found for node configuration {path}")
 
         lockfile = path.append(".locked")
         t = time()
@@ -83,10 +95,20 @@ class Config:
             for key in self.configs:
                 if key not in self.EXPECTED_CONFIGS and key not in self.ADDITIONNAL_CONFIGS:
                     raise IndexError(f"Unexpected config {key}")
-
-            for key in self.configs:
                 if self.configs[key] == "":
-                    self.configs[key] = None
+                    if key.endswith("_kwargs"):
+                        self.configs[key] = {}
+                    else:
+                        self.configs[key] = None
+                elif key.endswith("_path"):
+                    self.configs[key] = handle_path(self.configs[key], self.configs.get(f"{key}_fs", None))
+                elif key.endswith("_paths"):
+                    fss = self.configs.get(f"{key}_fss", None)
+                    if fss is not None:
+                        new_configs = []
+                        for element, fs in zip(self.configs[key], fss):
+                            new_configs.append(handle_path(element, fs))
+                        self.configs[key] = new_configs
 
             lockfile.rm(absent="ignore")
         except Exception as e:
@@ -153,7 +175,7 @@ class NodePublicConfig(Config):
     Used by `ifra.node.Node` and `ifra.central_server.NodeGate`
 
     Overloads \_\_eq\_\_ to allow for node configuration comparison. Two configurations are equal if all their
-    configuration values are equal, except *local_model_path* and *central_model_path* that can be different.
+    configuration values are equal, except *node_model_path* and *central_model_path* that can be different.
 
     Attributes
     ----------
@@ -190,28 +212,50 @@ class NodePublicConfig(Config):
     get_leaf: bool
         If True, will only consider the leaf ofthe tree to create rules. If False, every node of the tree will also be
         a rule.
-    local_model_path: TransparentPath
-        Path where the node is supposed to write its model. Should point to a csv file to which the central server have
+    node_model_path: TransparentPath
+        Path where the node is supposed to write its model. Should point to a csv file to which the aggregator has
         access.
-    central_model_path: TransparentPath
-        Path where the node is supposed to look for the central model. Should point to a csv file.
+    node_model_path_fs: TransparentPath
+        File system to use for local model file. Can be 'gcs', 'local' or "". If not specified, the json
+        file should still contain the key *node_model_path_fs*, but with value "".
+    id: Union[None, int, str]
+        Name or number of the node. Can not be "".
     dataprep: Union[str]
         Name of the dataprep to use. Can be "" or one of:\n
           * BinFeaturesDataPrep (see `ifra.datapreps.BinFeaturesDataPrep`)\n
         If not specified, will be set by central server. If not specified, the json  file should still contain the key
         *dataprep*, but with value "".
     dataprep_kwargs: dict
-        Keyword arguments for the dataprep. If not specified, will be set by central server. If not specified, the json
-        file should still contain the key *dataprep_kwargs*, but with value "".
-    id: Union[None, int, str]
-        Name or number of the node. If not specified, will be set by central server. If not specified, the json file
-        should still contain the key *id*, but with value "".
+        Keyword arguments for the `ifra.datapreps.Dataprep` init. If not specified, the json file should still contain
+        the key *dataprep_kwargs*, but with value "".
     fitter: str
         Fitter to use. Can be one of :\n
           * decisiontree_fitter (see `ifra.fitter.DecisionTreeFitter`)\n
+    fitter_kwargs: dict
+        Keyword arguments for the `ifra.fitters.Fitter` init. If not specified, the json file should still contain the
+        key *fitter_kwargs*, but with value "".
     updater: str
         Update method to be used by the node to take the central model into account. Can be one of:\n
           * adaboost_updater (see `ifra.updaters.AdaboostUpdater`)\n
+    updater_kwargs: dict
+        Keyword arguments for the `ifra.updaters.Updater` init. If not specified, the json file should still contain the
+        key *updater_kwargs*, but with value "".
+    thresholds_path: TransparentPath
+        Path to the json file to use to set `ruleskit.thresholds.Thresholds`. If not specified, the json
+        file should still contain the key *thresholds_path*, but with value "".
+    thresholds_path_fs: TransparentPath
+        File system to use for thresholds json file. Can be 'gcs', 'local' or "". If not specified, the json
+        file should still contain the key *thresholds_path_fs*, but with value "".
+    emitter_path: TransparentPath
+        Path to emitter json file. See `ifra.messenger.Emitter`
+    emitter_path_fs: TransparentPath
+        File system of path to emitter. Can be 'gcs', 'local' or "". If not specified, the json
+        file should still contain the key *emitter_path_fs*, but with value "".
+    central_model_path: TransparentPath
+        Path where the node is supposed to look for the central model.
+    central_model_path_fs: str
+        File system of central model path. Can be 'gcs', 'local' or "". If not specified, the json
+        file should still contain the key *central_model_path_fs*, but with value "".
     """
 
     EXPECTED_CONFIGS = [
@@ -224,39 +268,53 @@ class NodePublicConfig(Config):
         "stack_activation",
         "plot_data",
         "get_leaf",
-        "local_model_path",
-        "central_model_path",
+        "node_model_path",
+        "node_model_path_fs",
         "dataprep",
         "dataprep_kwargs",
         "id",
         "fitter",
-        "updater"
+        "fitter_kwargs",
+        "updater",
+        "updater_kwargs",
+        "thresholds_path",
+        "thresholds_path_fs",
+        "emitter_path",
+        "emitter_path_fs",
+        "central_model_path",
+        "central_model_path_fs"
     ]
 
-    def __init__(self, path: Union[str, Path, TransparentPath]):
-        super().__init__(path)
-        if isinstance(self.configs["local_model_path"], str):
-            self.configs["local_model_path"] = TransparentPath(self.configs["local_model_path"])
-        if isinstance(self.configs["central_model_path"], str):
-            self.configs["central_model_path"] = TransparentPath(self.configs["central_model_path"])
-
     def __eq__(self, other):
         if not isinstance(other, NodePublicConfig):
             return False
         for key in NodePublicConfig.EXPECTED_CONFIGS:
-            # Those parameters can change without problem
-            if key == "local_model_path" or key == "central_model_path" or key == "id":
+            # Those parameters can be different
+            if (
+                    key == "emitter_path"
+                    or key == "node_model_path"
+                    or key == "id"
+                    or key.endswith("_fs")
+                    or key.endswith("_fss")
+                    or key == "plot_data"
+            ):
                 continue
+            if key not in self.configs and key not in other.configs:
+                continue
+            if key == "thresholds_path":  # Can be either both "", or both not "" and different or both "" and equal
+                if (self.configs[key] == "" and other.configs[key] == "")\
+                        or (self.configs[key] != "" and other.configs[key] != ""):
+                    continue
             if self.configs[key] != other.configs[key]:
                 return False
         return True
 
 
-class CentralConfig(Config):
+class AggregatorConfig(Config):
     # noinspection PyUnresolvedReferences
-    """Overloads `Config`, corresponding to the central server configuration.
+    """Overloads `Config`, corresponding to the aggregator configuration.
 
-    Used by `ifra.central_server.CentralServer`
+    Used by `ifra.aggregator.Aggregator`
 
     Attributes
     ----------
@@ -265,37 +323,75 @@ class CentralConfig(Config):
     configs: dict
         see `Config`
 
-    max_coverage: float
-        Maximum coverage allowed for a rule of the node to be aggregated into the central mode
-    output_path: TransparentPath
-        Path where the central server will save its model at the end of the learning. Will also produce one output each
+    aggregated_model_path: TransparentPath
+        Path where the aggregator will save its model after each aggregations. Will also produce one output each
         time the model is updated. Should point to a csv file.
-    output_path_fs: str
+    aggregated_model_path_fs: str
         File system to use for learning outpout. Can be 'gcs', 'local' or ""
     min_number_of_new_models: int
         Minimum number of nodes that must have prodived a new model to trigger aggregation.
     aggregation: str
         Name of the aggregation method. Can be one of: \n
-          * adaboost_aggregation (see `ifra.aggregations.AdaBoostAggregation`)\n
+          * adaboost (see `ifra.aggregations.AdaBoostAggregation`)\n
+    aggregation_kwargs: dict
+        Keyword arguments for the `ifra.aggregations.Aggregation` init. If not specified, the json file should still
+        contain the key *aggregation_kwargs*, but with value "".
+    emitter_path: TransparentPath
+        Path to emitter json file. See `ifra.messenger.Emitter`
+    emitter_path_fs: TransparentPath
+        File system of path to emitter. Can be 'gcs', 'local' or "". If not specified, the json
+        file should still contain the key *emitter_path_fs*, but with value "".
     """
 
     EXPECTED_CONFIGS = [
-        "max_coverage",
-        "output_path",
-        "output_path_fs",
+        "aggregated_model_path",
+        "aggregated_model_path_fs",
         "min_number_of_new_models",
-        "aggregation"
+        "aggregation",
+        "aggregation_kwargs",
+        "emitter_path",
+        "emitter_path_fs",
     ]
 
-    def __init__(self, path: Union[str, Path, TransparentPath]):
-        super().__init__(path)
-        if isinstance(self.configs["output_path"], str):
-            if self.configs["output_path_fs"] is not None:
-                self.configs["output_path"] = TransparentPath(
-                    self.configs["output_path"], fs=self.configs["output_path_fs"]
-                )
-            else:
-                self.configs["output_path"] = TransparentPath(self.configs["output_path"])
+
+class CentralConfig(Config):
+    # noinspection PyUnresolvedReferences
+    """Overloads `Config`, corresponding to the aggregator configuration.
+
+    Used by `ifra.aggregator.Aggregator`
+
+    Attributes
+    ----------
+    path: TransparentPath
+        see `Config`
+    configs: dict
+        see `Config`
+
+    aggregated_model_path: TransparentPath
+        Path where the aggregator will save its model after each aggregations. Will also produce one output each
+        time the model is updated. Should point to a csv file.
+    aggregated_model_path_fs: str
+        File system to use for learning outpout. Can be 'gcs', 'local' or ""
+    central_model_path: TransparentPath
+        Path where the central model will save its model after each update. Will also produce one output each
+        time the model is updated. Should point to a csv file.
+    central_model_path_fs: str
+        File system to use for learning outpout. Can be 'gcs', 'local' or ""
+    emitter_path: TransparentPath
+        Path to emitter json file. See `ifra.messenger.Emitter`
+    emitter_path_fs: TransparentPath
+        File system of path to emitter. Can be 'gcs', 'local' or "". If not specified, the json
+        file should still contain the key *emitter_path_fs*, but with value "".
+    """
+
+    EXPECTED_CONFIGS = [
+        "aggregated_model_path",
+        "aggregated_model_path_fs",
+        "central_model_path",
+        "central_model_path_fs",
+        "emitter_path",
+        "emitter_path_fs",
+    ]
 
 
 class NodeDataConfig(Config):
@@ -318,10 +414,10 @@ class NodeDataConfig(Config):
     configs: dict
         see `Config`
 
-    x: TransparentPath
+    x_path: TransparentPath
         Path to the features file. If it is a csv, it MUST contain the index and columns (can be integers), and
         x_read_kwargs should contain index_col=0
-    y: TransparentPath
+    y_path: TransparentPath
         Path to the target (classes) file. If it is a csv, it MUST contain the index and columns (can be integers), and
         y_read_kwargs should contain index_col=0
     x_read_kwargs: Union[None, dict]
@@ -330,40 +426,67 @@ class NodeDataConfig(Config):
     y_read_kwargs: Union[None, dict]
         Keyword arguments to read the target file. If not specified, the json file should still contain the key
         *x_read_kwargs*, but with value "".
-    x_fs: str
+    x_path_fs: str
         File system to use for x file. Can be 'gcs', 'local' or ""
-    y_fs: str
+    y_path_fs: str
         File system to use for y file. Can be 'gcs', 'local' or ""
     dataprep_kwargs: dict
         Set by `ifra.node.Node`. Keyword arguments for the dataprep.
+    x_path_to_use: TransparentPath
+        Set by `ifra.node.Node.fit`. Path to the (datapreped and) copied features file used for learning.
+    y_path_to_use: TransparentPath
+        Set by `ifra.node.Node.fit`. Path to the (datapreped and) copied target file used for learning.
     """
 
-    EXPECTED_CONFIGS = ["x", "y", "x_read_kwargs", "y_read_kwargs", "x_fs", "y_fs"]
-    ADDITIONNAL_CONFIGS = {"dataprep_kwargs": {}}
+    EXPECTED_CONFIGS = ["x_path", "y_path", "x_read_kwargs", "y_read_kwargs", "x_path_fs", "y_path_fs"]
+    ADDITIONNAL_CONFIGS = {"dataprep_kwargs": {}, "x_path_to_use": None, "y_path_to_use": None}
 
-    def __init__(self, path: Union[str, Path, TransparentPath]):
+    def __init__(self, path: Optional[Union[str, Path, TransparentPath]] = None):
         super().__init__(path)
 
         if "index_col" in self.x_read_kwargs:
             if self.x_read_kwargs["index_col"] != 0:
-                raise ValueError("If specifying index_col, it should be 0, otherwise you will have problem when saving "
-                                 "intermediate data")
+                raise ValueError("If specifying index_col, it should be 0, otherwise you will have problems when saving"
+                                 " intermediate x learning copies")
         else:
             if self.x.suffix == ".csv":
-                raise ValueError("If file is a csv, then its read kwargs should contain index_col=0")
+                raise ValueError("If x file is a csv, then its read kwargs should contain 'index_col=0'")
 
         if "index_col" in self.y_read_kwargs:
             if self.y_read_kwargs["index_col"] != 0:
-                raise ValueError("If specifying index_col, it should be 0, otherwise you will have problem when saving "
-                                 "intermediate data")
+                raise ValueError("If specifying index_col, it should be 0, otherwise you will have problems when saving"
+                                 " intermediate y learning copies")
         else:
             if self.y.suffix == ".csv":
-                raise ValueError("If file is a csv, then its read kwargs should contain index_col=0")
+                raise ValueError("If y file is a csv, then its read kwargs should contain 'index_col=0'")
 
-        for item in self.configs:
-            if isinstance(self.configs[item], str):
-                fs = f"{item}_fs"
-                if fs in self.configs and self.configs[fs] is not None:
-                    self.configs[item] = TransparentPath(self.configs[item], fs=self.configs[fs])
-                else:
-                    self.configs[item] = TransparentPath(self.configs[item])
+
+class MonitoringConfig(Config):
+    # noinspection PyUnresolvedReferences
+    """Overloads `Config`, configuration for monitoring the learning.
+
+    Used by `ifra.monitoring.Monitor`
+
+    Attributes
+    ----------
+    path: TransparentPath
+        see `Config`
+    configs: dict
+        see `Config`
+
+    central_server_receiver_path: TransparentPath
+    central_server_receiver_path_fs: str
+    aggregator_receiver_path: TransparentPath
+    aggregator_receiver_path_fs: str
+    nodes_receivers_paths: List[TransparentPath]
+    nodes_receivers_paths_fss: List[str]
+    """
+
+    EXPECTED_CONFIGS = [
+        "central_server_receiver_path",
+        "central_server_receiver_path_fs",
+        "aggregator_receiver_path",
+        "aggregator_receiver_path_fs",
+        "nodes_receivers_paths",
+        "nodes_receivers_paths_fss",
+    ]
