@@ -6,6 +6,7 @@ import numpy as np
 from ruleskit import RuleSet
 from sklearn import tree
 from ruleskit.utils.rule_utils import extract_rules_from_tree
+from ruleskit import Rule
 import logging
 
 from .configs import NodePublicConfig, NodeDataConfig
@@ -19,13 +20,26 @@ class Fitter:
     def __init__(
         self,
         public_configs: NodePublicConfig,
-        data: NodeDataConfig,
+        **kwargs
     ):
-        self.public_configs = public_configs
-        self.paths = data
-        self.ruleset = None
+        """
 
-    def fit(self) -> RuleSet:
+        Parameters
+        ----------
+        public_configs: NodePublicConfig
+            `ifra.node.Node`'s *public_config*
+        data: NodeDataConfig
+            `ifra.node.Node`'s *data*
+        kwargs:
+            Any additionnal keyword argument that the overleading class accepts. Those arguments will become attributes.
+        """
+        self.public_configs = public_configs
+        self.ruleset = None
+        Rule.SET_THRESHOLDS(self.public_configs.thresholds_path)
+        for arg in kwargs:
+            setattr(self, arg, kwargs[arg])
+
+    def fit(self, x: np.ndarray, y: np.ndarray) -> RuleSet:
         """To be implemented in daughter class"""
         pass
 
@@ -34,14 +48,12 @@ class DecisionTreeFitter(Fitter):
 
     """Overloads the Fitter class. Fits a DecisionTreeFitter on some data.
 
-    Can be used by giving *decisiontree_fitter* as *fitter* configuration when creating a `ifra.node.Node`
+    Can be used by giving *decisiontree* as *fitter* configuration when creating a `ifra.node.Node`
 
     Attributes
     ----------
     public_configs: `ifra.configs.NodePublicConfig`
         The public configuration of the node using this fitter
-    data: NodeDataConfig
-        The data paths configuration of the node using this fitter
     ruleset: Union[None, RuleSet]
         Fitted ruleset, or None if fit not done yet
     tree: Union[None, DecisionTreeFitter]
@@ -51,26 +63,32 @@ class DecisionTreeFitter(Fitter):
     def __init__(
         self,
         public_configs: NodePublicConfig,
-        data: NodeDataConfig,
     ):
-        super().__init__(public_configs, data)
+        super().__init__(public_configs)
         self.tree = None
 
-    def fit(self) -> RuleSet:
-        """Fits the decision tree on the data pointed by `ifra.fitters.DecisionTreeFitter` *paths.x* and
-        `ifra.fitters.DecisionTreeFitter` *paths.y*, sets
+    def fit(self, x: np.ndarray, y: np.ndarray) -> RuleSet:
+        """Fits the decision tree on the data pointed by `ifra.fitters.DecisionTreeFitter` *data.x_path* and
+        `ifra.fitters.DecisionTreeFitter` *data.y_path*, sets
         `ifra.fitters.DecisionTreeFitter`*tree* saves it as a .dot, .svg and .joblib file in the same place
         the node will save its ruleset. Those files will be unique for each time the fit function is called.
         Also sets `ifra.fitters.DecisionTreeFitter` *ruleset* and returns it.
 
+        Parameters
+        ----------
+        x: np.ndarray
+            The features on which to learn
+        y: np.ndarray
+            The target to predict
+
         Returns
         -------
         RuleSet
             `ifra.fitters.DecisionTreeFitter` *ruleset*
         """
         self.make_fit(
-            self.paths.x.read(**self.paths.x_read_kwargs).values,
-            self.paths.y.read(**self.paths.y_read_kwargs).values,
+            x,
+            y,
             self.public_configs.max_depth,
             self.public_configs.get_leaf,
             self.public_configs.x_mins,
@@ -151,7 +169,7 @@ class DecisionTreeFitter(Fitter):
         if len(self.ruleset) > 0:
             # Compute each rule's activation vector, and the ruleset's if remember_activation, and will stack the
             # rules' if stack_activation is True
-            self.ruleset.calc_activation(x)
+            self.ruleset.fit(y=y, xs=x)
             # self.ruleset.check_duplicated_rules(self.ruleset.rules, name_or_index="name")
 
     def tree_to_graph(
@@ -166,12 +184,12 @@ class DecisionTreeFitter(Fitter):
         thetree = self.tree
         features_names = self.public_configs.features_names
         iteration = 0
-        name = self.public_configs.local_model_path.stem
-        path = self.public_configs.local_model_path.parent / f"{name}_{iteration}.dot"
+        name = self.public_configs.node_model_path.stem
+        path = self.public_configs.node_model_path.parent / f"{name}_{iteration}.dot"
 
         while path.is_file():
             iteration += 1
-            path = self.public_configs.local_model_path.parent / f"{name}_{iteration}.dot"
+            path = self.public_configs.node_model_path.parent / f"{name}_{iteration}.dot"
 
         with open(path, "w") as dotfile:
             tree.export_graphviz(
@@ -197,12 +215,12 @@ class DecisionTreeFitter(Fitter):
 
         thetree = self.tree
         iteration = 0
-        name = self.public_configs.local_model_path.stem
-        path = self.public_configs.local_model_path.parent / f"{name}_{iteration}.joblib"
+        name = self.public_configs.node_model_path.stem
+        path = self.public_configs.node_model_path.parent / f"{name}_{iteration}.joblib"
 
         while path.is_file():
             iteration += 1
-            path = self.public_configs.local_model_path.parent / f"{name}_{iteration}.joblib"
+            path = self.public_configs.node_model_path.parent / f"{name}_{iteration}.joblib"
 
         path = path.with_suffix(".joblib")
         joblib.dump(thetree, path)
